@@ -32,10 +32,34 @@ class ultraArm:
         data = None
         if self._serial_port.inWaiting() > 0:
             data = self._serial_port.read(self._serial_port.inWaiting())
+            queue_size = self._get_queue_size()
             if data != None:
-                data = str(data.decode())
-                if self.debug:
-                    print(data)
+                if 70 <= queue_size <= 139:
+                    data = str(data.decode())
+                    if self.debug:
+                        print(data)
+                elif 0 <= queue_size < 70:
+                    data = str(data.decode())
+                    if self.debug:
+                        print(data)
+                else:
+                    qs = 0
+                    while True:
+                        try:
+                            time.sleep(2)
+                            qs = self._get_queue_size()
+                            if self.debug:
+                                print('respone_size1:', qs)
+                            if qs < 70:
+                                time.sleep(1)
+                                qsize = self._get_queue_size()
+                                print('respone_size2:', qsize)
+                                break
+                                
+                        except Exception as e:
+                            print(e)
+                   
+               
 
     def _request(self, flag=""):
         """Get data from the robot"""
@@ -58,7 +82,7 @@ class ultraArm:
                     if a != None and a != "":
                         try:
                             all = list(map(float, a[astart + 1 : aend].split(",")))
-                            return all[:3]
+                            return all
                         except Exception:
                             print("received angles is not completed! Retry receive...")
                             count += 1
@@ -71,7 +95,7 @@ class ultraArm:
                     if c != None and c != "":
                         try:
                             all = list(map(float, c[cstart + 1 : cend].split(",")))
-                            return all[:3]
+                            return all
                         except Exception:
                             print("received coords is not completed! Retry receive...")
                             count += 1
@@ -213,11 +237,12 @@ class ultraArm:
 
         Args:
             degrees: a list of coords value(List[float]).
-                x : 0 ~ 270 mm
-                y : 0 ~ 270 mm
-                z : 0 ~ 125 mm
+                x : -260 ~ 300 mm
+                y : -300 ~ 300 mm
+                z : -70 ~ 135 mm
             speed : (int) 0-100 mm/s
         """
+        length = len(degrees)
         degrees = [degree for degree in degrees]
         command = ProtocolCode.COORDS_SET
         if degrees[0] is not None:
@@ -226,6 +251,9 @@ class ultraArm:
             command += " Y" + str(degrees[1])
         if degrees[2] is not None:
             command += " Z" + str(degrees[2])
+        if length == 4:
+            if degrees[3] is not None:
+                command += " E" + str(degrees[3])
         if speed > 0:
             command += " F" + str(speed)
         command += ProtocolCode.END
@@ -239,18 +267,18 @@ class ultraArm:
 
         Args:
             coord:
-                x : 0 ~ 270 mm
-                y : 0 ~ 270 mm
-                z : 0 ~ 125 mm
+                x : -260 ~ 300 mm
+                y : -300 ~ 300 mm
+                z : -70 ~ 135 mm
             speed : (int) 0-100 mm/s
         """
 
         command = ProtocolCode.COORDS_SET
         if id == "x" or id == "X":
             command += " X" + str(coord)
-        if id == "y" and id == "Y":
+        if id == "y" or id == "Y":
             command += " Y" + str(coord)
-        if id == "z" and id == "z":
+        if id == "z" or id == "Z":
             command += " Z" + str(coord)
         if speed > 0:
             command += " F" + str(speed)
@@ -314,8 +342,8 @@ class ultraArm:
 
         Args:
             state:
-                0 - close
-                1 - open
+                0 - open
+                1 - close
         """
         if state:
             command = ProtocolCode.GPIO_ON + ProtocolCode.END
@@ -371,12 +399,13 @@ class ultraArm:
         """Set single angle.
 
         Args:
-            id : joint (1/2/3)
+            id : joint (1/2/3/4)
 
             angle :
-                1 : -170° ~ +170°
-                2 : 0° ~ 90°
-                3 : 0° ~ 75°
+                1 : -150° ~ +170°
+                2 : -20° ~ 90°
+                3 : -5° ~ 60°
+                4 : -180° ~ + 180°
             speed : (int) 0-100 mm/s
         """
         command = ProtocolCode.SET_ANGLE
@@ -397,11 +426,15 @@ class ultraArm:
 
         Args:
             degrees: a list of angles value(List[float]).
-                joint1 : -170° ~ +170°
-                joint2 : 0° ~ 90°
-                joint3 : 0° ~ 75°
+            angle :
+                1 : -150° ~ +170°
+                2 : -20° ~ 90°
+                3 : -5° ~ 60°
+                4 : -180° ~ + 180°
+            speed : (int) 0-100 mm/s
             speed : (int) 0-100 mm/s
         """
+        length = len(degrees)
         degrees = [degree for degree in degrees]
         command = ProtocolCode.SET_ANGLES
         if degrees[0] is not None:
@@ -410,6 +443,9 @@ class ultraArm:
             command += " Y" + str(degrees[1])
         if degrees[2] is not None:
             command += " Z" + str(degrees[2])
+        if length == 4:
+            if degrees[3] is not None:
+                command += " E" + str(degrees[3])
         if speed > 0:
             command += " F" + str(speed)
         command += ProtocolCode.END
@@ -459,14 +495,18 @@ class ultraArm:
         """Start jog movement with coord
 
         Args:
-            axis : x/y/z
+            axis : 
+            1 : x
+            2 : y
+            3 : z
+            4 : θ
 
             direction:
                 0 : positive
                 1 : negative
             speed : (int) 0-100 mm/s
         """
-        command = ProtocolCode.JOG_COORD
+        command = ProtocolCode.JOG_COORD_
         if axis is not None:
             command += " j" + str(axis)
         if direction is not None:
